from datetime import datetime
import sys

class LogWriter(object):

    def __init__(self, log_file=sys.stdout):
        self.log_file = log_file

    def write_status(self, id, status):

        msg = ("%s -> %s, "
               "Generation: %s/%s, "
               "Validation set error: %f, "
               "Training set error: %f, "
               "Epoch: %d, "
               "Optimization Id: %s") % (
            datetime.now(),
            status.state,
            status.generation,
            status.total_generations,
            status.validation_set_error,
            status.training_set_error,
            status.epoch,
            id
        )
        self.write_string(msg)

    def write_string(self, msg):
        if isinstance(self.log_file, str):
            with open(self.log_file, mode='a', encoding='utf-8', buffering=1) as f:
                f.write(msg+'\n')
        else:
            self.log_file.write(msg+'\n')
            self.log_file.flush()



import sys
from contextlib import contextmanager


@contextmanager
def except_handler(exc_handler):
    sys.excepthook = exc_handler
    yield
    sys.excepthook = sys.__excepthook__


def exchandler(type, value, traceback):
    print(': '.join([str(type.__name__), str(value)]))


def validate_ranges(config):
    for r in config.input_ranges:
        if r.min > r.max:
            with except_handler(exchandler):
                raise ValueError(
                    "Maximum value must be greater than minimum value.")
    for r in config.output_ranges:
        if r.min > r.max:
            with except_handler(exchandler):
                raise ValueError(
                    "Maximum value must be greater than minimum value.")


def validate_train_keras(config):
    if len(config.hidden_layer_configs) == 0:
        with except_handler(exchandler):
            raise ValueError(
                "Number of hidden layers must be greater than zero.")
    if not isinstance(config.max_epoch, int):
        with except_handler(exchandler):
            raise ValueError("Number of epochs must be an integer.")
    if not(config.max_epoch > 0):
        with except_handler(exchandler):
            raise ValueError("Number of epochs must be greater than zero.")
    if not (config.validation_split >= 0 and config.validation_split <= 1):
        with except_handler(exchandler):
            raise ValueError("Validation split must be between 0 and 1.")
    for hlc in config.hidden_layer_configs:
        if hlc.neuron_count < 0:
            with except_handler(exchandler):
                raise ValueError(
                    "Number of neurons must be greater than zero.")
    for r in config.input_ranges:
        if r.min > r.max:
            with except_handler(exchandler):
                raise ValueError(
                    "Maximum value must be greater than minimum value.")
    for r in config.output_layer.ranges:
        if r.min > r.max:
            with except_handler(exchandler):
                raise ValueError(
                    "Maximum value must be greater than minimum value.")


def validate_predict_from_file_keras(config):
    return validate_ranges(config)


def validate_test_predict_array_keras(config):
    return validate_ranges(config)


def validate_optimize_keras(config):
    if config.engine_config.proc_timeout_miliseconds < 0:
        with except_handler(exchandler):
            raise ValueError("Values in seconds must be positive.")
    if(config.dropout.min > config.dropout.max):
        with except_handler(exchandler):
            raise ValueError(
                "Maximum value must be greater than minimum value.")
    if(config.hidden_layer_count_range.min >
       config.hidden_layer_count_range.max):
        with except_handler(exchandler):
            raise ValueError(
                "Maximum value must be greater than minimum value.")
    if not(config.hidden_layer_count_range.min > 0):
        with except_handler(exchandler):
            raise ValueError(
                "Number of hidden layers must be greater than zero.")
    if(config.neurons_per_layer.min > config.neurons_per_layer.max):
        with except_handler(exchandler):
            raise ValueError(
                "Maximum value must be greater than minimum value.")
    if not(config.neurons_per_layer.min > 0):
        with except_handler(exchandler):
            raise ValueError("Number of neurons must be greater than zero.")
    if not(config.validation_split >= 0 and config.validation_split <= 1):
        with except_handler(exchandler):
            raise ValueError("Validation split must be between 0 and 1.")
    if not(config.max_epoch > 0):
        with except_handler(exchandler):
            raise ValueError("Number of epochs must be greater than zero.")
    if not (isinstance(config.max_epoch, int)):
        with except_handler(exchandler):
            raise ValueError("Number of epochs must be an integer.")
    validate_ranges(config)


def validate_optimization(func):
    def decorator(*args, **kwargs):
        validate_optimize_keras(kwargs["config"])
        return func(*args, **kwargs)
    return decorator


def validate_training(func):
    def decorator(*args, **kwargs):
        validate_train_keras(kwargs["config"])
        return func(*args, **kwargs)
    return decorator


def validate_prediction_file(func):
    def decorator(*args, **kwargs):
        validate_predict_from_file_keras(kwargs["config"])
        return func(*args, **kwargs)
    return decorator


def validate_prediction_array(func):
    def decorator(*args, **kwargs):
        validate_test_predict_array_keras(kwargs["config"])
        return func(*args, **kwargs)
    return decorator



from tempfile import NamedTemporaryFile
from io import BytesIO
import os
import sys
import signal
import time
import shutil
import hashlib
from blackfox.rest import ApiException
from blackfox.configuration import Configuration
from blackfox.api_client import ApiClient
from blackfox.api.data_set_api import DataSetApi
from blackfox.api.network_api import NetworkApi
from blackfox.api.prediction_api import PredictionApi
from blackfox.api.training_api import TrainingApi
from blackfox.api.optimization_api import OptimizationApi
from blackfox.api.recurrent_optimization_api import RecurrentOptimizationApi
from blackfox.models.keras_optimization_config import KerasOptimizationConfig
from blackfox.log_writer import LogWriter
from blackfox.models.keras_recurrent_optimization_config import KerasRecurrentOptimizationConfig
from blackfox.models.keras_series_optimization_config import KerasSeriesOptimizationConfig
from blackfox.models.input_config import InputConfig
from blackfox.models.range import Range
from blackfox.validation import (validate_training,
                                 validate_optimization,
                                 validate_prediction_file,
                                 validate_prediction_array)


BUF_SIZE = 65536  # lets read stuff in 64kb chunks!


class BlackFox:

    def __init__(self, host="http://localhost:50476/"):
        self.host = host
        configuration = Configuration()
        configuration.host = host
        self.client = ApiClient(configuration)
        self.data_set_api = DataSetApi(self.client)
        self.network_api = NetworkApi(self.client)
        self.prediction_api = PredictionApi(self.client)
        self.training_api = TrainingApi(self.client)
        self.optimization_api = OptimizationApi(self.client)
        self.recurrent_optimization_api = RecurrentOptimizationApi(self.client)

    def __log_string(self, log_writer, msg):
        if log_writer is not None:
            log_writer.write_string(msg)

    def __log_status(self, log_writer, id, status):
        if log_writer is not None:
            log_writer.write_status(id, status)

    def upload_data_set(self, path):
        id = self.sha1(path)
        try:
            self.data_set_api.head(id)
        except ApiException as e:
            if e.status == 404:
                id = self.data_set_api.post(file=path)
            else:
                raise e
        return id

    def download_data_set(self, id, path):
        temp_path = self.data_set_api.get(id)
        shutil.move(temp_path, path)

    def upload_network(self, path):
        id = self.sha1(path)
        try:
            self.network_api.head(id)
        except ApiException as e:
            if e.status == 404:
                id = self.network_api.post(file=path)
            else:
                raise e
        return id

    def download_network(
        self, id, integrate_scaler=False,
        network_type='h5', path=None
    ):
        temp_path = self.network_api.get(
            id, integrate_scaler=integrate_scaler, network_type=network_type)
        if path is None:
            return open(temp_path, 'rb')
        else:
            shutil.move(temp_path, path)

    @validate_training
    def train_keras(
        self,
        config,
        integrate_scaler=False,
        network_type='h5',
        data_set_path=None,
        network_path=None
    ):
        """
        Train network

        :param KerasTrainingConfig config:
        :param str data_set_path:
        :param str nework_path:
        :return: TrainedNetwork
                If data_set_path is not None upload data set
                and sets config.dataset_id to new id.
                If network_path is not None 
                download network to given file.
        """
        if data_set_path is not None:
            config.dataset_id = self.upload_data_set(data_set_path)

        trained_network = self.training_api.post(value=config)

        if network_path is not None:
            self.download_network(
                trained_network.id,
                integrate_scaler=integrate_scaler,
                network_type=network_type,
                path=network_path)

        return trained_network

    @validate_prediction_file
    def predict_from_file_keras(
        self,
        config,
        network_path=None,
        data_set_path=None,
        result_path=None
    ):
        """
        Predict values and download results in file

        :param PredictionFileConfig config:
        :param str network_path:
        :param str data_set_path:
        :param str result_path:
        :return: str: result data set id
                If network_path is not None upload network,
                and sets config.network_id to new id.
                If data_set_path is not None upload data set,
                and sets config.data_set_id to new id.
                If result_path is not None download results
                to given file.
        """
        if network_path is not None:
            config.network_id = self.upload_network(network_path)
        if data_set_path is not None:
            config.data_set_id = self.upload_data_set(data_set_path)
        result_id = self.prediction_api.post_file(config=config)
        if result_path is not None:
            self.download_data_set(result_id, result_path)
        return result_id

    @validate_prediction_array
    def predict_from_array_keras(
        self,
        config,
        network_path=None
    ):
        """
        Predict values and return results

        :param PredictionArrayConfig config:
        :param str network_path:
        :return: list[list[float]]: 
                If network_path is not None upload network,
                and sets config.network_id to new id.
        """
        if network_path is not None:
            config.network_id = self.upload_network(network_path)
        results = self.prediction_api.post_array(config=config)
        return results

    def __get_ranges(self, data_set):
        ranges = []
        for row in data_set:
            for i, d in enumerate(row):
                if len(ranges) <= i or ranges[i] is None:
                    ranges.append(Range(d, d))
                else:
                    r = ranges[i]
                    r.min = min(r.min, d)
                    r.max = max(r.max, d)
        return ranges

    def __fill_inputs(self, inputs, data_set):
        if inputs is None or len(inputs) == 0:
            inputs = []
            for row in data_set:
                for i, d in enumerate(row):
                    if len(inputs) <= i or inputs[i] is None:
                        inputs.append(InputConfig(range=Range(d, d)))
                    else:
                        r = inputs[i].range
                        r.min = min(r.min, d)
                        r.max = max(r.max, d)
        else:
            for j in range(len(inputs)):
                inp = inputs[j]
                if inp.range is None:
                    inp.range = Range(float("inf"), float("-inf"))
                    for row in data_set:
                        d = row[j]
                        inp.range.min = min(inp.range.min, d)
                        inp.range.max = max(inp.range.max, d)
        return inputs

    def optimize_keras_sync(
        self,
        input_set=None,
        output_set=None,
        integrate_scaler=False,
        network_type='h5',
        data_set_path=None,
        config=KerasOptimizationConfig(),
        network_path=None,
        status_interval=5,
        log_writer=LogWriter()
    ):
        """
        Find optimal network for given problem.
        :param KerasOptimizationConfig config:
        :param str input_set:
        :param str output_set:
        :param bool integrate_scaler:
        :param str network_type:
        :param str data_set_path:
        :param str network_path:
        :param int status_interval:
        :param str log_writer:
        :return: (BytesIO, KerasOptimizedNetwork): byte array from network model, optimized network info
                If data_set_path is not None upload data set,
                and sets config.dataset_id to new id.
                If network_path is not None download network to given file.
                If log_writer is not None write to log file 
                every 5 seconds(status_interval)
        """
        return self.__optimize_keras_sync(
            is_series=False,
            input_set=input_set,
            output_set=output_set,
            integrate_scaler=integrate_scaler,
            network_type=network_type,
            data_set_path=data_set_path,
            config=config,
            network_path=network_path,
            status_interval=status_interval,
            log_writer=log_writer
        )

    def optimize_series_keras_sync(
        self,
        input_set=None,
        output_set=None,
        integrate_scaler=False,
        network_type='h5',
        data_set_path=None,
        config=KerasSeriesOptimizationConfig(),
        network_path=None,
        status_interval=5,
        log_writer=LogWriter()
    ):
        """
        Find optimal network for given problem.
        :param KerasSeriesOptimizationConfig config:
        :param str input_set:
        :param str output_set:
        :param bool integrate_scaler:
        :param str network_type:
        :param str data_set_path:
        :param str network_path:
        :param int status_interval:
        :param str log_writer:
        :return: (BytesIO, KerasOptimizedNetwork, dict): byte array from network model, optimized network info, metadata
                If data_set_path is not None upload data set,
                and sets config.dataset_id to new id.
                If network_path is not None download network to given file.
                If log_writer is not None write to log file 
                every 5 seconds(status_interval)
        """
        return self.__optimize_keras_sync(
            is_series=True,
            input_set=input_set,
            output_set=output_set,
            integrate_scaler=integrate_scaler,
            network_type=network_type,
            data_set_path=data_set_path,
            config=config,
            network_path=network_path,
            status_interval=status_interval,
            log_writer=log_writer
        )

    def __optimize_keras_sync(
        self,
        is_series=False,
        input_set=None,
        output_set=None,
        integrate_scaler=False,
        network_type='h5',
        data_set_path=None,
        config=None,
        network_path=None,
        status_interval=5,
        log_writer=LogWriter()
    ):
        print('Use CTRL + C to stop optimization')
        tmp_file = None
        if input_set is not None and output_set is not None:
            if type(input_set) is not list:
                input_set = input_set.tolist()
            if type(output_set) is not list:
                output_set = output_set.tolist()
            tmp_file = NamedTemporaryFile(delete=False)
            # input ranges
            config.inputs = self.__fill_inputs(config.inputs, input_set)
            # output ranges
            if config.output_ranges is None:
                config.output_ranges = self.__get_ranges(output_set)
            data_set = list(map(lambda x, y: (','.join(map(str, x)))+',' +
                                (','.join(map(str, y))), input_set, output_set))

            column_count = len(config.inputs) + len(config.output_ranges)
            column_range = range(0, column_count)
            headers = map(lambda i: 'column_'+str(i), column_range)
            data_set.insert(0, ','.join(headers))
            csv = '\n'.join(data_set)
            tmp_file.write(csv.encode("utf-8"))
            tmp_file.close()
            if data_set_path is not None:
                self.__log_string(log_writer, 'Ignoring data_set_path')
            data_set_path = str(tmp_file.name)

        if data_set_path is not None:
            if config.inputs is None:
                self.__log_string(log_writer, "config.inputs is None")
                return None, None, None
            if config.output_ranges is None:
                self.__log_string(log_writer, "config.output_ranges is None")
                return None, None, None
            if tmp_file is not None:
                self.__log_string(log_writer, "Uploading data set")
            else:
                self.__log_string(log_writer, "Uploading data set " + data_set_path)
            config.dataset_id = self.upload_data_set(data_set_path)

        if tmp_file is not None:
            os.remove(tmp_file.name)

        self.__log_string(log_writer, "Starting...")
        if is_series:
            id = self.optimization_api.post_series_async(config=config)
        else:
            id = self.optimization_api.post_async(config=config)

        def signal_handler(sig, frame):
            self.__log_string(log_writer, "Stopping optimization : "+id)
            if hasattr(log_writer, 'log_file') is False or log_writer.log_file is not sys.stdout:
                print("Stopping optimization : "+id)
            self.stop_optimization_keras(id)

        signal.signal(signal.SIGINT, signal_handler)

        running = True
        status = None
        while running:
            status = self.optimization_api.get_status_async(id)
            running = (status.state == 'Active')
            self.__log_status(log_writer, id, status)
            time.sleep(status_interval)

        if status.state == 'Finished' or status.state == 'Stopped':
            print('Optimization state: ', status.state)
            if status.network is not None and status.network.id is not None:
                self.__log_string(log_writer,
                         "Downloading network " +
                         status.network.id)
                network_stream = self.download_network(
                    status.network.id,
                    integrate_scaler=integrate_scaler,
                    network_type=network_type
                )
                data = network_stream.read()
                if network_path is not None:
                    self.__log_string(log_writer,
                             "Saving network " +
                             status.network.id + " to " + network_path)
                    with open(network_path, 'wb') as f:
                        f.write(data)
                byte_io = BytesIO(data)
                metadata = self.network_api.metadata(status.network.id)
                return byte_io, status.network, metadata
            else:
                return None, None, None

        elif status.state == 'Error':
            self.__log_string(log_writer, "Optimization error")
        else:
            self.__log_string(log_writer, "Unknown error")

        return None, None, None

    @validate_optimization
    def optimize_keras(
        self,
        config,
        data_set_path=None
    ):
        """
        Find optimal network for given problem async.

        :param KerasOptimizationConfig config:
        :param str data_set_path:
        :return: str: 
                If data_set_path is not None upload data set,
                and sets config.dataset_id to new id.
                Return optimization id.
        """
        # validate_optimize_keras(config)
        if data_set_path is not None:
            config.dataset_id = self.upload_data_set(data_set_path)
        return self.optimization_api.post_async(config=config)

    def get_optimization_status_keras(
        self,
        id,
        integrate_scaler=False,
        network_type='h5',
        network_path=None
    ):
        """
        Get status for optimization.

        :param KerasOptimizationConfig config:
        :param str network_path:
        :return: KerasOptimizationStatus: 
                If data_set_path is not None upload data set,
                and sets config.dataset_id to new id.
        """
        status = self.optimization_api.get_status_async(id)
        if (
            (status.state == 'Finished' or status.state == 'Stopped')
            and (network_path is not None)
        ):
            self.download_network(
                status.network.id,
                integrate_scaler=integrate_scaler,
                network_type=network_type,
                path=network_path)

        return status

    def cancel_optimization_keras(self, id):
        """
            Cancel optimization.

            :param str id:
            :return: None: 
                    Call get_optimization_status_keras to get status.
        """
        self.optimization_api.post_action_async(id, 'Cancel')

    def stop_optimization_keras(self, id, network_path=None):
        """
            Stop optimization

            :param str id:
            :param str network_path:
            :return: None: 
                    If network_path is not None download network to given file,
                    else call get_optimization_status_keras to get status
                    and download network.
        """
        self.optimization_api.post_action_async(id, 'Stop')
        if network_path is not None:
            state = 'Active'
            while state == 'Active':
                status = self.get_optimization_status_keras(id, network_path)
                state = status.state

    def optimize_recurrent_keras_sync(
        self,
        input_set=None,
        output_set=None,
        integrate_scaler=False,
        network_type='h5',
        data_set_path=None,
        config=None,
        network_path=None,
        status_interval=5,
        log_writer=LogWriter()
    ):
        """
        Find optimal network for given problem.
        :param KerasRecurrentOptimizationConfig config:
        :param str input_set:
        :param str output_set:
        :param bool integrate_scaler:
        :param str network_type:
        :param str data_set_path:
        :param str network_path:
        :param int status_interval:
        :param str log_writer:
        :return: (BytesIO, KerasRecurrentOptimizedNetwork, dict): byte array from network model, optimized network info, metadata
                If data_set_path is not None upload data set,
                and sets config.dataset_id to new id.
                If network_path is not None download network to given file.
                If log_writer is not None write to log file 
                every 5 seconds(status_interval)
        """
        print('Use CTRL + C to stop optimization')
        tmp_file = None
        if input_set is not None and output_set is not None:
            if type(input_set) is not list:
                input_set = input_set.tolist()
            if type(output_set) is not list:
                output_set = output_set.tolist()
            tmp_file = NamedTemporaryFile(delete=False)
            # input ranges
            config.inputs = self.__fill_inputs(config.inputs, input_set)
            # output ranges
            if config.output_ranges is None:
                config.output_ranges = self.__get_ranges(output_set)
            data_set = list(map(lambda x, y: (','.join(map(str, x)))+',' +
                                (','.join(map(str, y))), input_set, output_set))

            column_count = len(config.inputs) + len(config.output_ranges)
            column_range = range(0, column_count)
            headers = map(lambda i: 'column_'+str(i), column_range)
            data_set.insert(0, ','.join(headers))
            csv = '\n'.join(data_set)
            tmp_file.write(csv.encode("utf-8"))
            tmp_file.close()
            if data_set_path is not None:
                self.__log_string(log_writer, 'Ignoring data_set_path')
            data_set_path = str(tmp_file.name)

        if data_set_path is not None:
            if config.inputs is None:
                self.__log_string(log_writer, "config.inputs is None")
                return None, None, None
            if config.output_ranges is None:
                self.__log_string(log_writer, "config.output_ranges is None")
                return None, None, None
            if tmp_file is not None:
                self.__log_string(log_writer, "Uploading data set")
            else:
                self.__log_string(log_writer, "Uploading data set " +
                         data_set_path)
            config.dataset_id = self.upload_data_set(data_set_path)

        if tmp_file is not None:
            os.remove(tmp_file.name)

        self.__log_string(log_writer, "Starting...")
        id = self.recurrent_optimization_api.post(config=config)

        def signal_handler(sig, frame):
            self.__log_string(log_writer, "Stopping optimization : "+id)
            if hasattr(log_writer, 'log_file') is False or log_writer.log_file is not sys.stdout:
                print("Stopping optimization : "+id)
            self.stop_optimization_keras(id)

        signal.signal(signal.SIGINT, signal_handler)

        running = True
        status = None
        while running:
            status = self.recurrent_optimization_api.get_status(id)
            running = (status.state == 'Active')
            if log_writer is not None:
                self.__log_string(
                    log_writer,
                    ("%s -> %s, "
                     "Generation: %s/%s, "
                     "Validation set error: %f, "
                     "Training set error: %f, "
                     "Epoch: %d, "
                     "Optimization Id: %s") %
                    (
                        datetime.now(),
                        status.state,
                        status.generation,
                        status.total_generations,
                        status.validation_set_error,
                        status.training_set_error,
                        status.epoch,
                        id
                    )
                )
            time.sleep(status_interval)

        if status.state == 'Finished' or status.state == 'Stopped':
            print('Optimization state: ', status.state)
            if status.network is not None and status.network.id is not None:
                self.__log_string(log_writer,
                         "Downloading network " +
                         status.network.id)
                network_stream = self.download_network(
                    status.network.id,
                    integrate_scaler=integrate_scaler,
                    network_type=network_type
                )
                data = network_stream.read()
                if network_path is not None:
                    self.__log_string(log_writer,
                             "Saving network " +
                             status.network.id + " to " + network_path)
                    with open(network_path, 'wb') as f:
                        f.write(data)
                byte_io = BytesIO(data)
                metadata = self.network_api.metadata(status.network.id)
                return byte_io, status.network, metadata
            else:
                return None, None, None

        elif status.state == 'Error':
            self.__log_string(log_writer, "Optimization error")
        else:
            self.__log_string(log_writer, "Unknown error")

        return None, None, None

    def get_metadata(self, network_path):
        """
        Get network metadata.

        :param str or BytesIO network_path:
        :return: dict: 
        """
        id = None
        if isinstance(network_path, BytesIO):
            with NamedTemporaryFile(delete=False) as out:
                out.write(network_path.read())
                file_path = str(out.name)
            id = self.upload_network(file_path)
            os.remove(file_path)
        else:
            id = self.upload_network(network_path)

        return self.network_api.metadata(id)

    def convert_to(self, network_path, network_type, network_dst_path=None, integrate_scaler=False):
        id = None
        if isinstance(network_path, BytesIO):
            with NamedTemporaryFile(delete=False) as out:
                out.write(network_path.read())
                file_path = str(out.name)
            id = self.upload_network(file_path)
            os.remove(file_path)
        else:
            id = self.upload_network(network_path)
        stream = self.download_network(
            id,
            network_type=network_type,
            integrate_scaler=integrate_scaler,
            path=network_dst_path
        )
        if stream is not None:
            data = stream.read()
            byte_io = BytesIO(data)
            return byte_io

    def convert_to_onnx(
        self, network_path,
        network_dst_path=None, integrate_scaler=False
    ):
        self.convert_to(network_path, 'onnx', network_dst_path,
                        integrate_scaler=integrate_scaler)

    def convert_to_pb(
        self, network_path,
        network_dst_path=None, integrate_scaler=False
    ):
        self.convert_to(network_path, 'pb', network_dst_path,
                        integrate_scaler=integrate_scaler)

    def sha1(self, path):
        sha1 = hashlib.sha1()
        try:
            with open(path, 'rb') as f:
                while True:
                    data = f.read(BUF_SIZE)
                    if not data:
                        break
                    sha1.update(data)

        except IOError:
            print("File " + path + " doesn't exist.")

        return sha1.hexdigest()



# coding: utf-8

# flake8: noqa

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

# import apis into sdk package
from blackfox.api.data_set_api import DataSetApi
from blackfox.api.network_api import NetworkApi
from blackfox.api.optimization_api import OptimizationApi
from blackfox.api.prediction_api import PredictionApi
from blackfox.api.recurrent_optimization_api import RecurrentOptimizationApi
from blackfox.api.training_api import TrainingApi

# import ApiClient
from blackfox.api_client import ApiClient
from blackfox.configuration import Configuration
# import models into sdk package
from blackfox.models.convergency_criterion import ConvergencyCriterion
from blackfox.models.input_config import InputConfig
from blackfox.models.input_window_config import InputWindowConfig
from blackfox.models.input_window_range_config import InputWindowRangeConfig
from blackfox.models.keras_hidden_layer_config import KerasHiddenLayerConfig
from blackfox.models.keras_layer_config import KerasLayerConfig
from blackfox.models.keras_optimization_config import KerasOptimizationConfig
from blackfox.models.keras_optimization_status import KerasOptimizationStatus
from blackfox.models.keras_optimized_network import KerasOptimizedNetwork
from blackfox.models.keras_recurrent_hidden_layer_config import KerasRecurrentHiddenLayerConfig
from blackfox.models.keras_recurrent_optimization_config import KerasRecurrentOptimizationConfig
from blackfox.models.keras_recurrent_optimization_status import KerasRecurrentOptimizationStatus
from blackfox.models.keras_recurrent_optimized_network import KerasRecurrentOptimizedNetwork
from blackfox.models.keras_series_optimization_config import KerasSeriesOptimizationConfig
from blackfox.models.keras_series_training_config import KerasSeriesTrainingConfig
from blackfox.models.keras_training_config import KerasTrainingConfig
from blackfox.models.optimization_engine_config import OptimizationEngineConfig
from blackfox.models.output_window_config import OutputWindowConfig
from blackfox.models.prediction_array_config import PredictionArrayConfig
from blackfox.models.prediction_file_config import PredictionFileConfig
from blackfox.models.range import Range
from blackfox.models.recurrent_optimization_engine_config import RecurrentOptimizationEngineConfig
from blackfox.models.trained_network import TrainedNetwork
from blackfox.black_fox import BlackFox
from blackfox.log_writer import LogWriter
from blackfox.csv_log_writer import CsvLogWriter


# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.keras_recurrent_optimized_network import KerasRecurrentOptimizedNetwork  # noqa: F401,E501


class KerasRecurrentOptimizationStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'state': 'str',
        'generation': 'int',
        'total_generations': 'int',
        'validation_set_error': 'float',
        'training_set_error': 'float',
        'epoch': 'int',
        'network': 'KerasRecurrentOptimizedNetwork'
    }

    attribute_map = {
        'state': 'state',
        'generation': 'generation',
        'total_generations': 'totalGenerations',
        'validation_set_error': 'validationSetError',
        'training_set_error': 'trainingSetError',
        'epoch': 'epoch',
        'network': 'network'
    }

    def __init__(self, state=None, generation=None, total_generations=None, validation_set_error=None, training_set_error=None, epoch=None, network=None):  # noqa: E501
        """KerasRecurrentOptimizationStatus - a model defined in Swagger"""  # noqa: E501

        self._state = None
        self._generation = None
        self._total_generations = None
        self._validation_set_error = None
        self._training_set_error = None
        self._epoch = None
        self._network = None
        self.discriminator = None

        if state is not None:
            self.state = state
        if generation is not None:
            self.generation = generation
        if total_generations is not None:
            self.total_generations = total_generations
        if validation_set_error is not None:
            self.validation_set_error = validation_set_error
        if training_set_error is not None:
            self.training_set_error = training_set_error
        if epoch is not None:
            self.epoch = epoch
        if network is not None:
            self.network = network

    @property
    def state(self):
        """Gets the state of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The state of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this KerasRecurrentOptimizationStatus.


        :param state: The state of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: str
        """
        allowed_values = ["Active", "Finished", "Stopped", "Error"]  # noqa: E501
        if state not in allowed_values:
            raise ValueError(
                "Invalid value for `state` ({0}), must be one of {1}"  # noqa: E501
                .format(state, allowed_values)
            )

        self._state = state

    @property
    def generation(self):
        """Gets the generation of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The generation of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: int
        """
        return self._generation

    @generation.setter
    def generation(self, generation):
        """Sets the generation of this KerasRecurrentOptimizationStatus.


        :param generation: The generation of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: int
        """

        self._generation = generation

    @property
    def total_generations(self):
        """Gets the total_generations of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The total_generations of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: int
        """
        return self._total_generations

    @total_generations.setter
    def total_generations(self, total_generations):
        """Sets the total_generations of this KerasRecurrentOptimizationStatus.


        :param total_generations: The total_generations of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: int
        """

        self._total_generations = total_generations

    @property
    def validation_set_error(self):
        """Gets the validation_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The validation_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: float
        """
        return self._validation_set_error

    @validation_set_error.setter
    def validation_set_error(self, validation_set_error):
        """Sets the validation_set_error of this KerasRecurrentOptimizationStatus.


        :param validation_set_error: The validation_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: float
        """

        self._validation_set_error = validation_set_error

    @property
    def training_set_error(self):
        """Gets the training_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The training_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: float
        """
        return self._training_set_error

    @training_set_error.setter
    def training_set_error(self, training_set_error):
        """Sets the training_set_error of this KerasRecurrentOptimizationStatus.


        :param training_set_error: The training_set_error of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: float
        """

        self._training_set_error = training_set_error

    @property
    def epoch(self):
        """Gets the epoch of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The epoch of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: int
        """
        return self._epoch

    @epoch.setter
    def epoch(self, epoch):
        """Sets the epoch of this KerasRecurrentOptimizationStatus.


        :param epoch: The epoch of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: int
        """

        self._epoch = epoch

    @property
    def network(self):
        """Gets the network of this KerasRecurrentOptimizationStatus.  # noqa: E501


        :return: The network of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :rtype: KerasRecurrentOptimizedNetwork
        """
        return self._network

    @network.setter
    def network(self, network):
        """Sets the network of this KerasRecurrentOptimizationStatus.


        :param network: The network of this KerasRecurrentOptimizationStatus.  # noqa: E501
        :type: KerasRecurrentOptimizedNetwork
        """

        self._network = network

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasRecurrentOptimizationStatus):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from blackfox.api.data_set_api import DataSetApi
from blackfox.api.network_api import NetworkApi
from blackfox.api.optimization_api import OptimizationApi
from blackfox.api.prediction_api import PredictionApi
from blackfox.api.recurrent_optimization_api import RecurrentOptimizationApi
from blackfox.api.training_api import TrainingApi



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class ConvergencyCriterion(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'number_of_latest_generations': 'int',
        'percentage_of_tolerance': 'float'
    }

    attribute_map = {
        'number_of_latest_generations': 'numberOfLatestGenerations',
        'percentage_of_tolerance': 'percentageOfTolerance'
    }

    def __init__(self, number_of_latest_generations=10, percentage_of_tolerance=5):  # noqa: E501
        """ConvergencyCriterion - a model defined in Swagger"""  # noqa: E501

        self._number_of_latest_generations = None
        self._percentage_of_tolerance = None
        self.discriminator = None

        if number_of_latest_generations is not None:
            self.number_of_latest_generations = number_of_latest_generations
        if percentage_of_tolerance is not None:
            self.percentage_of_tolerance = percentage_of_tolerance

    @property
    def number_of_latest_generations(self):
        """Gets the number_of_latest_generations of this ConvergencyCriterion.  # noqa: E501


        :return: The number_of_latest_generations of this ConvergencyCriterion.  # noqa: E501
        :rtype: int
        """
        return self._number_of_latest_generations

    @number_of_latest_generations.setter
    def number_of_latest_generations(self, number_of_latest_generations):
        """Sets the number_of_latest_generations of this ConvergencyCriterion.


        :param number_of_latest_generations: The number_of_latest_generations of this ConvergencyCriterion.  # noqa: E501
        :type: int
        """

        self._number_of_latest_generations = number_of_latest_generations

    @property
    def percentage_of_tolerance(self):
        """Gets the percentage_of_tolerance of this ConvergencyCriterion.  # noqa: E501


        :return: The percentage_of_tolerance of this ConvergencyCriterion.  # noqa: E501
        :rtype: float
        """
        return self._percentage_of_tolerance

    @percentage_of_tolerance.setter
    def percentage_of_tolerance(self, percentage_of_tolerance):
        """Sets the percentage_of_tolerance of this ConvergencyCriterion.


        :param percentage_of_tolerance: The percentage_of_tolerance of this ConvergencyCriterion.  # noqa: E501
        :type: float
        """

        self._percentage_of_tolerance = percentage_of_tolerance

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConvergencyCriterion):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.keras_hidden_layer_config import KerasHiddenLayerConfig  # noqa: F401,E501
from blackfox.models.keras_layer_config import KerasLayerConfig  # noqa: F401,E501
from blackfox.models.range import Range  # noqa: F401,E501


class KerasTrainingConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'batch_size': 'int',
        'dataset_id': 'str',
        'input_ranges': 'list[Range]',
        'output_layer': 'KerasLayerConfig',
        'hidden_layer_configs': 'list[KerasHiddenLayerConfig]',
        'training_algorithm': 'str',
        'max_epoch': 'int',
        'cross_validation': 'bool',
        'validation_split': 'float',
        'random_seed': 'int'
    }

    attribute_map = {
        'batch_size': 'batchSize',
        'dataset_id': 'datasetId',
        'input_ranges': 'inputRanges',
        'output_layer': 'outputLayer',
        'hidden_layer_configs': 'hiddenLayerConfigs',
        'training_algorithm': 'trainingAlgorithm',
        'max_epoch': 'maxEpoch',
        'cross_validation': 'crossValidation',
        'validation_split': 'validationSplit',
        'random_seed': 'randomSeed'
    }

    def __init__(self, batch_size=None, dataset_id=None, input_ranges=None, output_layer=None, hidden_layer_configs=None, training_algorithm=None, max_epoch=None, cross_validation=None, validation_split=None, random_seed=None):  # noqa: E501
        """KerasTrainingConfig - a model defined in Swagger"""  # noqa: E501

        self._batch_size = None
        self._dataset_id = None
        self._input_ranges = None
        self._output_layer = None
        self._hidden_layer_configs = None
        self._training_algorithm = None
        self._max_epoch = None
        self._cross_validation = None
        self._validation_split = None
        self._random_seed = None
        self.discriminator = None

        if batch_size is not None:
            self.batch_size = batch_size
        if dataset_id is not None:
            self.dataset_id = dataset_id
        if input_ranges is not None:
            self.input_ranges = input_ranges
        if output_layer is not None:
            self.output_layer = output_layer
        if hidden_layer_configs is not None:
            self.hidden_layer_configs = hidden_layer_configs
        if training_algorithm is not None:
            self.training_algorithm = training_algorithm
        self.max_epoch = max_epoch
        if cross_validation is not None:
            self.cross_validation = cross_validation
        self.validation_split = validation_split
        if random_seed is not None:
            self.random_seed = random_seed

    @property
    def batch_size(self):
        """Gets the batch_size of this KerasTrainingConfig.  # noqa: E501


        :return: The batch_size of this KerasTrainingConfig.  # noqa: E501
        :rtype: int
        """
        return self._batch_size

    @batch_size.setter
    def batch_size(self, batch_size):
        """Sets the batch_size of this KerasTrainingConfig.


        :param batch_size: The batch_size of this KerasTrainingConfig.  # noqa: E501
        :type: int
        """

        self._batch_size = batch_size

    @property
    def dataset_id(self):
        """Gets the dataset_id of this KerasTrainingConfig.  # noqa: E501


        :return: The dataset_id of this KerasTrainingConfig.  # noqa: E501
        :rtype: str
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this KerasTrainingConfig.


        :param dataset_id: The dataset_id of this KerasTrainingConfig.  # noqa: E501
        :type: str
        """

        self._dataset_id = dataset_id

    @property
    def input_ranges(self):
        """Gets the input_ranges of this KerasTrainingConfig.  # noqa: E501


        :return: The input_ranges of this KerasTrainingConfig.  # noqa: E501
        :rtype: list[Range]
        """
        return self._input_ranges

    @input_ranges.setter
    def input_ranges(self, input_ranges):
        """Sets the input_ranges of this KerasTrainingConfig.


        :param input_ranges: The input_ranges of this KerasTrainingConfig.  # noqa: E501
        :type: list[Range]
        """

        self._input_ranges = input_ranges

    @property
    def output_layer(self):
        """Gets the output_layer of this KerasTrainingConfig.  # noqa: E501


        :return: The output_layer of this KerasTrainingConfig.  # noqa: E501
        :rtype: KerasLayerConfig
        """
        return self._output_layer

    @output_layer.setter
    def output_layer(self, output_layer):
        """Sets the output_layer of this KerasTrainingConfig.


        :param output_layer: The output_layer of this KerasTrainingConfig.  # noqa: E501
        :type: KerasLayerConfig
        """

        self._output_layer = output_layer

    @property
    def hidden_layer_configs(self):
        """Gets the hidden_layer_configs of this KerasTrainingConfig.  # noqa: E501


        :return: The hidden_layer_configs of this KerasTrainingConfig.  # noqa: E501
        :rtype: list[KerasHiddenLayerConfig]
        """
        return self._hidden_layer_configs

    @hidden_layer_configs.setter
    def hidden_layer_configs(self, hidden_layer_configs):
        """Sets the hidden_layer_configs of this KerasTrainingConfig.


        :param hidden_layer_configs: The hidden_layer_configs of this KerasTrainingConfig.  # noqa: E501
        :type: list[KerasHiddenLayerConfig]
        """

        self._hidden_layer_configs = hidden_layer_configs

    @property
    def training_algorithm(self):
        """Gets the training_algorithm of this KerasTrainingConfig.  # noqa: E501


        :return: The training_algorithm of this KerasTrainingConfig.  # noqa: E501
        :rtype: str
        """
        return self._training_algorithm

    @training_algorithm.setter
    def training_algorithm(self, training_algorithm):
        """Sets the training_algorithm of this KerasTrainingConfig.


        :param training_algorithm: The training_algorithm of this KerasTrainingConfig.  # noqa: E501
        :type: str
        """
        allowed_values = ["SGD", "RMSprop", "Adagrad", "Adadelta", "Adam", "Adamax", "Nadam"]  # noqa: E501
        if training_algorithm not in allowed_values:
            raise ValueError(
                "Invalid value for `training_algorithm` ({0}), must be one of {1}"  # noqa: E501
                .format(training_algorithm, allowed_values)
            )

        self._training_algorithm = training_algorithm

    @property
    def max_epoch(self):
        """Gets the max_epoch of this KerasTrainingConfig.  # noqa: E501


        :return: The max_epoch of this KerasTrainingConfig.  # noqa: E501
        :rtype: int
        """
        return self._max_epoch

    @max_epoch.setter
    def max_epoch(self, max_epoch):
        """Sets the max_epoch of this KerasTrainingConfig.


        :param max_epoch: The max_epoch of this KerasTrainingConfig.  # noqa: E501
        :type: int
        """
        if max_epoch is None:
            raise ValueError("Invalid value for `max_epoch`, must not be `None`")  # noqa: E501
        if max_epoch is not None and max_epoch > 4294967295:  # noqa: E501
            raise ValueError("Invalid value for `max_epoch`, must be a value less than or equal to `4294967295`")  # noqa: E501
        if max_epoch is not None and max_epoch < 1:  # noqa: E501
            raise ValueError("Invalid value for `max_epoch`, must be a value greater than or equal to `1`")  # noqa: E501

        self._max_epoch = max_epoch

    @property
    def cross_validation(self):
        """Gets the cross_validation of this KerasTrainingConfig.  # noqa: E501


        :return: The cross_validation of this KerasTrainingConfig.  # noqa: E501
        :rtype: bool
        """
        return self._cross_validation

    @cross_validation.setter
    def cross_validation(self, cross_validation):
        """Sets the cross_validation of this KerasTrainingConfig.


        :param cross_validation: The cross_validation of this KerasTrainingConfig.  # noqa: E501
        :type: bool
        """

        self._cross_validation = cross_validation

    @property
    def validation_split(self):
        """Gets the validation_split of this KerasTrainingConfig.  # noqa: E501


        :return: The validation_split of this KerasTrainingConfig.  # noqa: E501
        :rtype: float
        """
        return self._validation_split

    @validation_split.setter
    def validation_split(self, validation_split):
        """Sets the validation_split of this KerasTrainingConfig.


        :param validation_split: The validation_split of this KerasTrainingConfig.  # noqa: E501
        :type: float
        """
        if validation_split is None:
            raise ValueError("Invalid value for `validation_split`, must not be `None`")  # noqa: E501
        if validation_split is not None and validation_split > 1.0:  # noqa: E501
            raise ValueError("Invalid value for `validation_split`, must be a value less than or equal to `1.0`")  # noqa: E501
        if validation_split is not None and validation_split < 0.0:  # noqa: E501
            raise ValueError("Invalid value for `validation_split`, must be a value greater than or equal to `0.0`")  # noqa: E501

        self._validation_split = validation_split

    @property
    def random_seed(self):
        """Gets the random_seed of this KerasTrainingConfig.  # noqa: E501


        :return: The random_seed of this KerasTrainingConfig.  # noqa: E501
        :rtype: int
        """
        return self._random_seed

    @random_seed.setter
    def random_seed(self, random_seed):
        """Sets the random_seed of this KerasTrainingConfig.


        :param random_seed: The random_seed of this KerasTrainingConfig.  # noqa: E501
        :type: int
        """

        self._random_seed = random_seed

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasTrainingConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class KerasRecurrentHiddenLayerConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'recurrent_activation_function': 'str',
        'recurrent_dropout': 'float',
        'neuron_count': 'int',
        'activation_function': 'str',
        'dropout': 'float'
    }

    attribute_map = {
        'recurrent_activation_function': 'recurrentActivationFunction',
        'recurrent_dropout': 'recurrentDropout',
        'neuron_count': 'neuronCount',
        'activation_function': 'activationFunction',
        'dropout': 'dropout'
    }

    def __init__(self, recurrent_activation_function=None, recurrent_dropout=None, neuron_count=None, activation_function=None, dropout=None):  # noqa: E501
        """KerasRecurrentHiddenLayerConfig - a model defined in Swagger"""  # noqa: E501

        self._recurrent_activation_function = None
        self._recurrent_dropout = None
        self._neuron_count = None
        self._activation_function = None
        self._dropout = None
        self.discriminator = None

        if recurrent_activation_function is not None:
            self.recurrent_activation_function = recurrent_activation_function
        if recurrent_dropout is not None:
            self.recurrent_dropout = recurrent_dropout
        if neuron_count is not None:
            self.neuron_count = neuron_count
        if activation_function is not None:
            self.activation_function = activation_function
        if dropout is not None:
            self.dropout = dropout

    @property
    def recurrent_activation_function(self):
        """Gets the recurrent_activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501


        :return: The recurrent_activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :rtype: str
        """
        return self._recurrent_activation_function

    @recurrent_activation_function.setter
    def recurrent_activation_function(self, recurrent_activation_function):
        """Sets the recurrent_activation_function of this KerasRecurrentHiddenLayerConfig.


        :param recurrent_activation_function: The recurrent_activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :type: str
        """
        allowed_values = ["SoftMax", "Elu", "Selu", "SoftPlus", "SoftSign", "ReLu", "TanH", "Sigmoid", "HardSigmoid", "Linear"]  # noqa: E501
        if recurrent_activation_function not in allowed_values:
            raise ValueError(
                "Invalid value for `recurrent_activation_function` ({0}), must be one of {1}"  # noqa: E501
                .format(recurrent_activation_function, allowed_values)
            )

        self._recurrent_activation_function = recurrent_activation_function

    @property
    def recurrent_dropout(self):
        """Gets the recurrent_dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501


        :return: The recurrent_dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :rtype: float
        """
        return self._recurrent_dropout

    @recurrent_dropout.setter
    def recurrent_dropout(self, recurrent_dropout):
        """Sets the recurrent_dropout of this KerasRecurrentHiddenLayerConfig.


        :param recurrent_dropout: The recurrent_dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :type: float
        """

        self._recurrent_dropout = recurrent_dropout

    @property
    def neuron_count(self):
        """Gets the neuron_count of this KerasRecurrentHiddenLayerConfig.  # noqa: E501


        :return: The neuron_count of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :rtype: int
        """
        return self._neuron_count

    @neuron_count.setter
    def neuron_count(self, neuron_count):
        """Sets the neuron_count of this KerasRecurrentHiddenLayerConfig.


        :param neuron_count: The neuron_count of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :type: int
        """

        self._neuron_count = neuron_count

    @property
    def activation_function(self):
        """Gets the activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501


        :return: The activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :rtype: str
        """
        return self._activation_function

    @activation_function.setter
    def activation_function(self, activation_function):
        """Sets the activation_function of this KerasRecurrentHiddenLayerConfig.


        :param activation_function: The activation_function of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :type: str
        """
        allowed_values = ["SoftMax", "Elu", "Selu", "SoftPlus", "SoftSign", "ReLu", "TanH", "Sigmoid", "HardSigmoid", "Linear"]  # noqa: E501
        if activation_function not in allowed_values:
            raise ValueError(
                "Invalid value for `activation_function` ({0}), must be one of {1}"  # noqa: E501
                .format(activation_function, allowed_values)
            )

        self._activation_function = activation_function

    @property
    def dropout(self):
        """Gets the dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501


        :return: The dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :rtype: float
        """
        return self._dropout

    @dropout.setter
    def dropout(self, dropout):
        """Sets the dropout of this KerasRecurrentHiddenLayerConfig.


        :param dropout: The dropout of this KerasRecurrentHiddenLayerConfig.  # noqa: E501
        :type: float
        """

        self._dropout = dropout

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasRecurrentHiddenLayerConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from datetime import datetime
import sys


class CsvLogWriter(object):

    def __init__(self, log_file=sys.stdout, only_change=True):
        self.log_file = log_file
        self.write_string(
            'Time,Status,Generation,Total generations,Validation set error,Training set error,Epoch,Optimization Id')
        self.only_change = only_change
        self.previous_generation = -1

    def write_status(self, id, status):
        if self.only_change == False or self.previous_generation != status.generation:
            msg = ("%s,%s,%s,%s,%f,%f,%d,%s") % (
                datetime.now(),
                status.state,
                status.generation,
                status.total_generations,
                status.validation_set_error,
                status.training_set_error,
                status.epoch,
                id
            )
            self.write_string(msg)
            self.previous_generation = status.generation

    def write_string(self, msg):
        if isinstance(self.log_file, str):
            with open(self.log_file, mode='a', encoding='utf-8', buffering=1) as f:
                f.write(msg+'\n')
        else:
            self.log_file.write(msg+'\n')
            self.log_file.flush()



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from blackfox.api_client import ApiClient


class NetworkApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get(self, id, **kwargs):  # noqa: E501
        """Download nework file (*.h5)  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Nework Id (required)
        :param bool integrate_scaler: 
        :param str network_type: 
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def get_with_http_info(self, id, **kwargs):  # noqa: E501
        """Download nework file (*.h5)  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Nework Id (required)
        :param bool integrate_scaler: 
        :param str network_type: 
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id', 'integrate_scaler', 'network_type']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'integrate_scaler' in params:
            query_params.append(('integrateScaler', params['integrate_scaler']))  # noqa: E501
        if 'network_type' in params:
            query_params.append(('networkType', params['network_type']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/octet-stream'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/Network/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='file',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def head(self, id, **kwargs):  # noqa: E501
        """Check if h5 file exist  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.head(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: File hash(sha1) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.head_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.head_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def head_with_http_info(self, id, **kwargs):  # noqa: E501
        """Check if h5 file exist  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.head_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: File hash(sha1) (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method head" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `head`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/Network/{id}', 'HEAD',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def metadata(self, id, **kwargs):  # noqa: E501
        """Get h5 file metadata  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.metadata(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: File hash(sha1) (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.metadata_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.metadata_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def metadata_with_http_info(self, id, **kwargs):  # noqa: E501
        """Get h5 file metadata  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.metadata_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: File hash(sha1) (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method metadata" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `metadata`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['text/plain', 'application/json', 'text/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/Network/{id}/Metadata', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def post(self, **kwargs):  # noqa: E501
        """Upload h5 file  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file file:
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.post_with_http_info(**kwargs)  # noqa: E501
            return data

    def post_with_http_info(self, **kwargs):  # noqa: E501
        """Upload h5 file  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file file:
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['file']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method post" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}
        if 'file' in params:
            local_var_files['file'] = params['file']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['text/plain', 'application/json', 'text/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['multipart/form-data'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/api/Network', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='str',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.input_config import InputConfig  # noqa: F401,E501
from blackfox.models.optimization_engine_config import OptimizationEngineConfig  # noqa: F401,E501
from blackfox.models.range import Range  # noqa: F401,E501


class KerasOptimizationConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dropout': 'Range',
        'batch_size': 'int',
        'dataset_id': 'str',
        'inputs': 'list[InputConfig]',
        'output_ranges': 'list[Range]',
        'problem_type': 'str',
        'hidden_layer_count_range': 'Range',
        'neurons_per_layer': 'Range',
        'training_algorithms': 'list[str]',
        'activation_functions': 'list[str]',
        'max_epoch': 'int',
        'cross_validation': 'bool',
        'validation_split': 'float',
        'random_seed': 'int',
        'engine_config': 'OptimizationEngineConfig'
    }

    attribute_map = {
        'dropout': 'dropout',
        'batch_size': 'batchSize',
        'dataset_id': 'datasetId',
        'inputs': 'inputs',
        'output_ranges': 'outputRanges',
        'problem_type': 'problemType',
        'hidden_layer_count_range': 'hiddenLayerCountRange',
        'neurons_per_layer': 'neuronsPerLayer',
        'training_algorithms': 'trainingAlgorithms',
        'activation_functions': 'activationFunctions',
        'max_epoch': 'maxEpoch',
        'cross_validation': 'crossValidation',
        'validation_split': 'validationSplit',
        'random_seed': 'randomSeed',
        'engine_config': 'engineConfig'
    }

    def __init__(self, 
        dropout=Range(min=0, max=25),
        batch_size=32,
        dataset_id=None,
        inputs=None,
        output_ranges=None,
        problem_type='Regression',
        hidden_layer_count_range=Range(min=1, max=15),
        neurons_per_layer=Range(min=1, max=10),
        training_algorithms=["SGD", "RMSprop", "Adagrad", "Adadelta", "Adam", "Adamax", "Nadam"],
        activation_functions=["SoftMax", "Elu", "Selu", "SoftPlus",
                                      "SoftSign", "ReLu", "TanH", "Sigmoid",
                                      "HardSigmoid", "Linear"],
        max_epoch=3000,
        cross_validation=False,
        validation_split=0.2,
        random_seed=300,
        engine_config=OptimizationEngineConfig()
    ):  # noqa: E501
        """KerasOptimizationConfig - a model defined in Swagger"""  # noqa: E501

        self._dropout = None
        self._batch_size = None
        self._dataset_id = None
        self._inputs = None
        self._output_ranges = None
        self._problem_type = None
        self._hidden_layer_count_range = None
        self._neurons_per_layer = None
        self._training_algorithms = None
        self._activation_functions = None
        self._max_epoch = None
        self._cross_validation = None
        self._validation_split = None
        self._random_seed = None
        self._engine_config = None
        self.discriminator = None

        if dropout is not None:
            self.dropout = dropout
        if batch_size is not None:
            self.batch_size = batch_size
        if dataset_id is not None:
            self.dataset_id = dataset_id
        if inputs is not None:
            self.inputs = inputs
        if output_ranges is not None:
            self.output_ranges = output_ranges
        if problem_type is not None:
            self.problem_type = problem_type
        if hidden_layer_count_range is not None:
            self.hidden_layer_count_range = hidden_layer_count_range
        if neurons_per_layer is not None:
            self.neurons_per_layer = neurons_per_layer
        if training_algorithms is not None:
            self.training_algorithms = training_algorithms
        if activation_functions is not None:
            self.activation_functions = activation_functions
        self.max_epoch = max_epoch
        if cross_validation is not None:
            self.cross_validation = cross_validation
        self.validation_split = validation_split
        if random_seed is not None:
            self.random_seed = random_seed
        if engine_config is not None:
            self.engine_config = engine_config

    @property
    def dropout(self):
        """Gets the dropout in percent.
        Min value: 0
        Max value: 100

        :return: The dropout of this KerasOptimizationConfig.  # noqa: E501
        :rtype: Range
        """
        return self._dropout

    @dropout.setter
    def dropout(self, dropout):
        """Sets the dropout of this KerasOptimizationConfig.


        :param dropout: The dropout of this KerasOptimizationConfig.  # noqa: E501
        :type: Range
        """

        self._dropout = dropout

    @property
    def batch_size(self):
        """Gets the batch_size of this KerasOptimizationConfig.  # noqa: E501


        :return: The batch_size of this KerasOptimizationConfig.  # noqa: E501
        :rtype: int
        """
        return self._batch_size

    @batch_size.setter
    def batch_size(self, batch_size):
        """Sets the batch_size of this KerasOptimizationConfig.


        :param batch_size: The batch_size of this KerasOptimizationConfig.  # noqa: E501
        :type: int
        """

        self._batch_size = batch_size

    @property
    def dataset_id(self):
        """Gets the dataset_id of this KerasOptimizationConfig.  # noqa: E501


        :return: The dataset_id of this KerasOptimizationConfig.  # noqa: E501
        :rtype: str
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this KerasOptimizationConfig.


        :param dataset_id: The dataset_id of this KerasOptimizationConfig.  # noqa: E501
        :type: str
        """

        self._dataset_id = dataset_id

    @property
    def inputs(self):
        """Gets the inputs of this KerasOptimizationConfig.  # noqa: E501


        :return: The inputs of this KerasOptimizationConfig.  # noqa: E501
        :rtype: list[InputConfig]
        """
        return self._inputs

    @inputs.setter
    def inputs(self, inputs):
        """Sets the inputs of this KerasOptimizationConfig.


        :param inputs: The inputs of this KerasOptimizationConfig.  # noqa: E501
        :type: list[InputConfig]
        """

        self._inputs = inputs

    @property
    def output_ranges(self):
        """Gets the output_ranges of this KerasOptimizationConfig.  # noqa: E501


        :return: The output_ranges of this KerasOptimizationConfig.  # noqa: E501
        :rtype: list[Range]
        """
        return self._output_ranges

    @output_ranges.setter
    def output_ranges(self, output_ranges):
        """Sets the output_ranges of this KerasOptimizationConfig.


        :param output_ranges: The output_ranges of this KerasOptimizationConfig.  # noqa: E501
        :type: list[Range]
        """

        self._output_ranges = output_ranges

    @property
    def problem_type(self):
        """Gets the problem_type of this KerasOptimizationConfig.  # noqa: E501


        :return: The problem_type of this KerasOptimizationConfig.  # noqa: E501
        :rtype: str
        """
        return self._problem_type

    @problem_type.setter
    def problem_type(self, problem_type):
        """Sets the problem_type of this KerasOptimizationConfig.


        :param problem_type: The problem_type of this KerasOptimizationConfig.  # noqa: E501
        :type: str
        """
        allowed_values = ["Regression", "BinaryClassification", "MultiClassClassification"]  # noqa: E501
        if problem_type not in allowed_values:
            raise ValueError(
                "Invalid value for `problem_type` ({0}), must be one of {1}"  # noqa: E501
                .format(problem_type, allowed_values)
            )

        self._problem_type = problem_type

    @property
    def hidden_layer_count_range(self):
        """Gets the hidden_layer_count_range of this KerasOptimizationConfig.  # noqa: E501


        :return: The hidden_layer_count_range of this KerasOptimizationConfig.  # noqa: E501
        :rtype: Range
        """
        return self._hidden_layer_count_range

    @hidden_layer_count_range.setter
    def hidden_layer_count_range(self, hidden_layer_count_range):
        """Sets the hidden_layer_count_range of this KerasOptimizationConfig.


        :param hidden_layer_count_range: The hidden_layer_count_range of this KerasOptimizationConfig.  # noqa: E501
        :type: Range
        """

        self._hidden_layer_count_range = hidden_layer_count_range

    @property
    def neurons_per_layer(self):
        """Gets the neurons_per_layer of this KerasOptimizationConfig.  # noqa: E501


        :return: The neurons_per_layer of this KerasOptimizationConfig.  # noqa: E501
        :rtype: Range
        """
        return self._neurons_per_layer

    @neurons_per_layer.setter
    def neurons_per_layer(self, neurons_per_layer):
        """Sets the neurons_per_layer of this KerasOptimizationConfig.


        :param neurons_per_layer: The neurons_per_layer of this KerasOptimizationConfig.  # noqa: E501
        :type: Range
        """

        self._neurons_per_layer = neurons_per_layer

    @property
    def training_algorithms(self):
        """Gets the training_algorithms of this KerasOptimizationConfig.  # noqa: E501


        :return: The training_algorithms of this KerasOptimizationConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._training_algorithms

    @training_algorithms.setter
    def training_algorithms(self, training_algorithms):
        """Sets the training_algorithms of this KerasOptimizationConfig.


        :param training_algorithms: The training_algorithms of this KerasOptimizationConfig.  # noqa: E501
        :type: list[str]
        """
        allowed_values = ["SGD", "RMSprop", "Adagrad", "Adadelta", "Adam", "Adamax", "Nadam"]  # noqa: E501
        if not set(training_algorithms).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `training_algorithms` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(training_algorithms) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._training_algorithms = training_algorithms

    @property
    def activation_functions(self):
        """Gets the activation_functions of this KerasOptimizationConfig.  # noqa: E501


        :return: The activation_functions of this KerasOptimizationConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._activation_functions

    @activation_functions.setter
    def activation_functions(self, activation_functions):
        """Sets the activation_functions of this KerasOptimizationConfig.


        :param activation_functions: The activation_functions of this KerasOptimizationConfig.  # noqa: E501
        :type: list[str]
        """
        allowed_values = ["SoftMax", "Elu", "Selu", "SoftPlus", "SoftSign", "ReLu", "TanH", "Sigmoid", "HardSigmoid", "Linear"]  # noqa: E501
        if not set(activation_functions).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `activation_functions` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(activation_functions) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._activation_functions = activation_functions

    @property
    def max_epoch(self):
        """Gets the max_epoch of this KerasOptimizationConfig.  # noqa: E501


        :return: The max_epoch of this KerasOptimizationConfig.  # noqa: E501
        :rtype: int
        """
        return self._max_epoch

    @max_epoch.setter
    def max_epoch(self, max_epoch):
        """Sets the max_epoch of this KerasOptimizationConfig.


        :param max_epoch: The max_epoch of this KerasOptimizationConfig.  # noqa: E501
        :type: int
        """
        if max_epoch is None:
            raise ValueError("Invalid value for `max_epoch`, must not be `None`")  # noqa: E501
        if max_epoch is not None and max_epoch > 4294967295:  # noqa: E501
            raise ValueError("Invalid value for `max_epoch`, must be a value less than or equal to `4294967295`")  # noqa: E501
        if max_epoch is not None and max_epoch < 1:  # noqa: E501
            raise ValueError("Invalid value for `max_epoch`, must be a value greater than or equal to `1`")  # noqa: E501

        self._max_epoch = max_epoch

    @property
    def cross_validation(self):
        """Gets the cross_validation of this KerasOptimizationConfig.  # noqa: E501


        :return: The cross_validation of this KerasOptimizationConfig.  # noqa: E501
        :rtype: bool
        """
        return self._cross_validation

    @cross_validation.setter
    def cross_validation(self, cross_validation):
        """Sets the cross_validation of this KerasOptimizationConfig.


        :param cross_validation: The cross_validation of this KerasOptimizationConfig.  # noqa: E501
        :type: bool
        """

        self._cross_validation = cross_validation

    @property
    def validation_split(self):
        """Gets the validation_split of this KerasOptimizationConfig.  # noqa: E501


        :return: The validation_split of this KerasOptimizationConfig.  # noqa: E501
        :rtype: float
        """
        return self._validation_split

    @validation_split.setter
    def validation_split(self, validation_split):
        """Sets the validation_split of this KerasOptimizationConfig.


        :param validation_split: The validation_split of this KerasOptimizationConfig.  # noqa: E501
        :type: float
        """
        if validation_split is None:
            raise ValueError("Invalid value for `validation_split`, must not be `None`")  # noqa: E501
        if validation_split is not None and validation_split > 1.0:  # noqa: E501
            raise ValueError("Invalid value for `validation_split`, must be a value less than or equal to `1.0`")  # noqa: E501
        if validation_split is not None and validation_split < 0.0:  # noqa: E501
            raise ValueError("Invalid value for `validation_split`, must be a value greater than or equal to `0.0`")  # noqa: E501

        self._validation_split = validation_split

    @property
    def random_seed(self):
        """Gets the random_seed of this KerasOptimizationConfig.  # noqa: E501


        :return: The random_seed of this KerasOptimizationConfig.  # noqa: E501
        :rtype: int
        """
        return self._random_seed

    @random_seed.setter
    def random_seed(self, random_seed):
        """Sets the random_seed of this KerasOptimizationConfig.


        :param random_seed: The random_seed of this KerasOptimizationConfig.  # noqa: E501
        :type: int
        """

        self._random_seed = random_seed

    @property
    def engine_config(self):
        """Gets the engine_config of this KerasOptimizationConfig.  # noqa: E501


        :return: The engine_config of this KerasOptimizationConfig.  # noqa: E501
        :rtype: OptimizationEngineConfig
        """
        return self._engine_config

    @engine_config.setter
    def engine_config(self, engine_config):
        """Sets the engine_config of this KerasOptimizationConfig.


        :param engine_config: The engine_config of this KerasOptimizationConfig.  # noqa: E501
        :type: OptimizationEngineConfig
        """

        self._engine_config = engine_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasOptimizationConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TrainedNetwork(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'epoch_count': 'int',
        'errors_on_validation_set': 'list[float]',
        'errors_on_training_set': 'list[float]'
    }

    attribute_map = {
        'id': 'id',
        'epoch_count': 'epochCount',
        'errors_on_validation_set': 'errorsOnValidationSet',
        'errors_on_training_set': 'errorsOnTrainingSet'
    }

    def __init__(self, id=None, epoch_count=None, errors_on_validation_set=None, errors_on_training_set=None):  # noqa: E501
        """TrainedNetwork - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._epoch_count = None
        self._errors_on_validation_set = None
        self._errors_on_training_set = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if epoch_count is not None:
            self.epoch_count = epoch_count
        if errors_on_validation_set is not None:
            self.errors_on_validation_set = errors_on_validation_set
        if errors_on_training_set is not None:
            self.errors_on_training_set = errors_on_training_set

    @property
    def id(self):
        """Gets the id of this TrainedNetwork.  # noqa: E501


        :return: The id of this TrainedNetwork.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this TrainedNetwork.


        :param id: The id of this TrainedNetwork.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def epoch_count(self):
        """Gets the epoch_count of this TrainedNetwork.  # noqa: E501


        :return: The epoch_count of this TrainedNetwork.  # noqa: E501
        :rtype: int
        """
        return self._epoch_count

    @epoch_count.setter
    def epoch_count(self, epoch_count):
        """Sets the epoch_count of this TrainedNetwork.


        :param epoch_count: The epoch_count of this TrainedNetwork.  # noqa: E501
        :type: int
        """

        self._epoch_count = epoch_count

    @property
    def errors_on_validation_set(self):
        """Gets the errors_on_validation_set of this TrainedNetwork.  # noqa: E501


        :return: The errors_on_validation_set of this TrainedNetwork.  # noqa: E501
        :rtype: list[float]
        """
        return self._errors_on_validation_set

    @errors_on_validation_set.setter
    def errors_on_validation_set(self, errors_on_validation_set):
        """Sets the errors_on_validation_set of this TrainedNetwork.


        :param errors_on_validation_set: The errors_on_validation_set of this TrainedNetwork.  # noqa: E501
        :type: list[float]
        """

        self._errors_on_validation_set = errors_on_validation_set

    @property
    def errors_on_training_set(self):
        """Gets the errors_on_training_set of this TrainedNetwork.  # noqa: E501


        :return: The errors_on_training_set of this TrainedNetwork.  # noqa: E501
        :rtype: list[float]
        """
        return self._errors_on_training_set

    @errors_on_training_set.setter
    def errors_on_training_set(self, errors_on_training_set):
        """Sets the errors_on_training_set of this TrainedNetwork.


        :param errors_on_training_set: The errors_on_training_set of this TrainedNetwork.  # noqa: E501
        :type: list[float]
        """

        self._errors_on_training_set = errors_on_training_set

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TrainedNetwork):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.range import Range  # noqa: F401,E501


class KerasLayerConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'activation_function': 'str',
        'ranges': 'list[Range]'
    }

    attribute_map = {
        'activation_function': 'activationFunction',
        'ranges': 'ranges'
    }

    def __init__(self, activation_function=None, ranges=None):  # noqa: E501
        """KerasLayerConfig - a model defined in Swagger"""  # noqa: E501

        self._activation_function = None
        self._ranges = None
        self.discriminator = None

        if activation_function is not None:
            self.activation_function = activation_function
        if ranges is not None:
            self.ranges = ranges

    @property
    def activation_function(self):
        """Gets the activation_function of this KerasLayerConfig.  # noqa: E501


        :return: The activation_function of this KerasLayerConfig.  # noqa: E501
        :rtype: str
        """
        return self._activation_function

    @activation_function.setter
    def activation_function(self, activation_function):
        """Sets the activation_function of this KerasLayerConfig.


        :param activation_function: The activation_function of this KerasLayerConfig.  # noqa: E501
        :type: str
        """
        allowed_values = ["SoftMax", "Elu", "Selu", "SoftPlus", "SoftSign", "ReLu", "TanH", "Sigmoid", "HardSigmoid", "Linear"]  # noqa: E501
        if activation_function not in allowed_values:
            raise ValueError(
                "Invalid value for `activation_function` ({0}), must be one of {1}"  # noqa: E501
                .format(activation_function, allowed_values)
            )

        self._activation_function = activation_function

    @property
    def ranges(self):
        """Gets the ranges of this KerasLayerConfig.  # noqa: E501


        :return: The ranges of this KerasLayerConfig.  # noqa: E501
        :rtype: list[Range]
        """
        return self._ranges

    @ranges.setter
    def ranges(self, ranges):
        """Sets the ranges of this KerasLayerConfig.


        :param ranges: The ranges of this KerasLayerConfig.  # noqa: E501
        :type: list[Range]
        """

        self._ranges = ranges

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasLayerConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.range import Range  # noqa: F401,E501


class InputWindowRangeConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'window': 'Range',
        'shift': 'Range',
        'step': 'Range',
        'aggregation_types': 'list[str]'
    }

    attribute_map = {
        'window': 'window',
        'shift': 'shift',
        'step': 'step',
        'aggregation_types': 'aggregationTypes'
    }

    def __init__(self, window=None, shift=None, step=None, aggregation_types=['Avg', 'Sum', 'None']):  # noqa: E501
        """InputWindowRangeConfig - a model defined in Swagger"""  # noqa: E501

        self._window = None
        self._shift = None
        self._step = None
        self._aggregation_types = None
        self.discriminator = None

        if window is not None:
            self.window = window
        if shift is not None:
            self.shift = shift
        if step is not None:
            self.step = step
        if aggregation_types is not None:
            self.aggregation_types = aggregation_types

    @property
    def window(self):
        """Gets the window of this InputWindowRangeConfig.  # noqa: E501


        :return: The window of this InputWindowRangeConfig.  # noqa: E501
        :rtype: Range
        """
        return self._window

    @window.setter
    def window(self, window):
        """Sets the window of this InputWindowRangeConfig.


        :param window: The window of this InputWindowRangeConfig.  # noqa: E501
        :type: Range
        """

        self._window = window

    @property
    def shift(self):
        """Gets the shift of this InputWindowRangeConfig.  # noqa: E501


        :return: The shift of this InputWindowRangeConfig.  # noqa: E501
        :rtype: Range
        """
        return self._shift

    @shift.setter
    def shift(self, shift):
        """Sets the shift of this InputWindowRangeConfig.


        :param shift: The shift of this InputWindowRangeConfig.  # noqa: E501
        :type: Range
        """

        self._shift = shift

    @property
    def step(self):
        """Gets the step of this InputWindowRangeConfig.  # noqa: E501


        :return: The step of this InputWindowRangeConfig.  # noqa: E501
        :rtype: Range
        """
        return self._step

    @step.setter
    def step(self, step):
        """Sets the step of this InputWindowRangeConfig.


        :param step: The step of this InputWindowRangeConfig.  # noqa: E501
        :type: Range
        """

        self._step = step

    @property
    def aggregation_types(self):
        """Gets the aggregation_types of this InputWindowRangeConfig.  # noqa: E501


        :return: The aggregation_types of this InputWindowRangeConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._aggregation_types

    @aggregation_types.setter
    def aggregation_types(self, aggregation_types):
        """Sets the aggregation_types of this InputWindowRangeConfig.


        :param aggregation_types: The aggregation_types of this InputWindowRangeConfig.  # noqa: E501
        :type: list[str]
        """
        allowed_values = ["Avg", "Sum", "None"]  # noqa: E501
        if not set(aggregation_types).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `aggregation_types` [{0}], must be a subset of [{1}]"  # noqa: E501
                .format(", ".join(map(str, set(aggregation_types) - set(allowed_values))),  # noqa: E501
                        ", ".join(map(str, allowed_values)))
            )

        self._aggregation_types = aggregation_types

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InputWindowRangeConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.keras_recurrent_hidden_layer_config import KerasRecurrentHiddenLayerConfig  # noqa: F401,E501


class KerasRecurrentOptimizedNetwork(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'hidden_layers': 'list[KerasRecurrentHiddenLayerConfig]',
        'training_algorithm': 'str',
        'output_layer_activation_function': 'str'
    }

    attribute_map = {
        'id': 'id',
        'hidden_layers': 'hiddenLayers',
        'training_algorithm': 'trainingAlgorithm',
        'output_layer_activation_function': 'outputLayerActivationFunction'
    }

    def __init__(self, id=None, hidden_layers=None, training_algorithm=None, output_layer_activation_function=None):  # noqa: E501
        """KerasRecurrentOptimizedNetwork - a model defined in Swagger"""  # noqa: E501

        self._id = None
        self._hidden_layers = None
        self._training_algorithm = None
        self._output_layer_activation_function = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if hidden_layers is not None:
            self.hidden_layers = hidden_layers
        if training_algorithm is not None:
            self.training_algorithm = training_algorithm
        if output_layer_activation_function is not None:
            self.output_layer_activation_function = output_layer_activation_function

    @property
    def id(self):
        """Gets the id of this KerasRecurrentOptimizedNetwork.  # noqa: E501


        :return: The id of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this KerasRecurrentOptimizedNetwork.


        :param id: The id of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def hidden_layers(self):
        """Gets the hidden_layers of this KerasRecurrentOptimizedNetwork.  # noqa: E501


        :return: The hidden_layers of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :rtype: list[KerasRecurrentHiddenLayerConfig]
        """
        return self._hidden_layers

    @hidden_layers.setter
    def hidden_layers(self, hidden_layers):
        """Sets the hidden_layers of this KerasRecurrentOptimizedNetwork.


        :param hidden_layers: The hidden_layers of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :type: list[KerasRecurrentHiddenLayerConfig]
        """

        self._hidden_layers = hidden_layers

    @property
    def training_algorithm(self):
        """Gets the training_algorithm of this KerasRecurrentOptimizedNetwork.  # noqa: E501


        :return: The training_algorithm of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :rtype: str
        """
        return self._training_algorithm

    @training_algorithm.setter
    def training_algorithm(self, training_algorithm):
        """Sets the training_algorithm of this KerasRecurrentOptimizedNetwork.


        :param training_algorithm: The training_algorithm of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :type: str
        """
        allowed_values = ["SGD", "RMSprop", "Adagrad", "Adadelta", "Adam", "Adamax", "Nadam"]  # noqa: E501
        if training_algorithm not in allowed_values:
            raise ValueError(
                "Invalid value for `training_algorithm` ({0}), must be one of {1}"  # noqa: E501
                .format(training_algorithm, allowed_values)
            )

        self._training_algorithm = training_algorithm

    @property
    def output_layer_activation_function(self):
        """Gets the output_layer_activation_function of this KerasRecurrentOptimizedNetwork.  # noqa: E501


        :return: The output_layer_activation_function of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :rtype: str
        """
        return self._output_layer_activation_function

    @output_layer_activation_function.setter
    def output_layer_activation_function(self, output_layer_activation_function):
        """Sets the output_layer_activation_function of this KerasRecurrentOptimizedNetwork.


        :param output_layer_activation_function: The output_layer_activation_function of this KerasRecurrentOptimizedNetwork.  # noqa: E501
        :type: str
        """
        allowed_values = ["SoftMax", "Elu", "Selu", "SoftPlus", "SoftSign", "ReLu", "TanH", "Sigmoid", "HardSigmoid", "Linear"]  # noqa: E501
        if output_layer_activation_function not in allowed_values:
            raise ValueError(
                "Invalid value for `output_layer_activation_function` ({0}), must be one of {1}"  # noqa: E501
                .format(output_layer_activation_function, allowed_values)
            )

        self._output_layer_activation_function = output_layer_activation_function

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KerasRecurrentOptimizedNetwork):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    BlackFox

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from blackfox.models.range import Range  # noqa: F401,E501


class PredictionFileConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'data_set_id': 'str',
        'network_id': 'str',
        'input_ranges': 'list[Range]',
        'output_ranges': 'list[Range]'
    }

    attribute_map = {
        'data_set_id': 'dataSetId',
        'network_id': 'networkId',
        'input_ranges': 'inputRanges',
        'output_ranges': 'outputRanges'
    }

    def __init__(self, data_set_id=None, network_id=None, input_ranges=None, output_ranges=None):  # noqa: E501
        """PredictionFileConfig - a model defined in Swagger"""  # noqa: E501

        self._data_set_id = None
        self._network_id = None
        self._input_ranges = None
        self._output_ranges = None
        self.discriminator = None

        if data_set_id is not None:
            self.data_set_id = data_set_id
        if network_id is not None:
            self.network_id = network_id
        if input_ranges is not None:
            self.input_ranges = input_ranges
        if output_ranges is not None:
            self.output_ranges = output_ranges

    @property
    def data_set_id(self):
        """Gets the data_set_id of this PredictionFileConfig.  # noqa: E501


        :return: The data_set_id of this PredictionFileConfig.  # noqa: E501
        :rtype: str
        """
        return self._data_set_id

    @data_set_id.setter
    def data_set_id(self, data_set_id):
        """Sets the data_set_id of this PredictionFileConfig.


        :param data_set_id: The data_set_id of this PredictionFileConfig.  # noqa: E501
        :type: str
        """

        self._data_set_id = data_set_id

    @property
    def network_id(self):
        """Gets the network_id of this PredictionFileConfig.  # noqa: E501


        :return: The network_id of this PredictionFileConfig.  # noqa: E501
        :rtype: str
        """
        return self._network_id

    @network_id.setter
    def network_id(self, network_id):
        """Sets the network_id of this PredictionFileConfig.


        :param network_id: The network_id of this PredictionFileConfig.  # noqa: E501
        :type: str
        """

        self._network_id = network_id

    @property
    def input_ranges(self):
        """Gets the input_ranges of this PredictionFileConfig.  # noqa: E501


        :return: The input_ranges of this PredictionFileConfig.  # noqa: E501
        :rtype: list[Range]
        """
        return self._input_ranges

    @input_ranges.setter
    def input_ranges(self, input_ranges):
        """Sets the input_ranges of this PredictionFileConfig.


        :param input_ranges: The input_ranges of this PredictionFileConfig.  # noqa: E501
        :type: list[Range]
        """

        self._input_ranges = input_ranges

    @property
    def output_ranges(self):
        """Gets the output_ranges of this PredictionFileConfig.  # noqa: E501


        :return: The output_ranges of this PredictionFileConfig.  # noqa: E501
        :rtype: list[Range]
        """
        return self._output_ranges

    @output_ranges.setter
    def output_ranges(self, output_ranges):
        """Sets the output_ranges of this PredictionFileConfig.


        :param output_ranges: The output_ranges of this PredictionFileConfig.  # noqa: E501
        :type: list[Range]
        """

        self._output_ranges = output_ranges

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PredictionFileConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
